/* Bear Markup Language (BML) parsing routines
 *  By Charles Shiflett
 *
 *  routines to parse bml files. (html files with database tags)
 *
 * Basic format is <!--#bear query string (parsed by cli library)
 *                  with variable insertion through %var% tag
 *
 *  for instance <!--#bear insert inventory=1 type=%type color=red-->
 *  would be an example string. The rest of BML is standard html.
 *
 * The functional interface is to call parse_bml(filename)
 *  which returns a list parsed into html (ptr[2*i]) and non html ptr[2*1+1].
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "cgi.h"

FILE* open_file(char* filename) {
	FILE *file;
	if ( !filename || !(*filename)) {
		printf("Error: No filename passed to bml parser\n"); 
		return(0);
	}
	if (strstr(filename, "..") || strstr(filename, "//")) {
		printf("Error: Filename may not contain .. or //\n");
		return(0);
	}
	if (*filename == '/') {
		printf("Error: Filename may not be preceeded by a /\n");
		return(0);
	}

	file=fopen(filename, "r");

	if (!file)
		perror("parse_bml could not open file");

	return file;
}

static char** query_string_ptr;

void add_query_string(char* query_string) {
	static int i;
	static char **query_string_old=0;

	if (query_string_old != query_string_ptr) {
		query_string_old=query_string_ptr;
		i=0;
	}

	if ( i > 255) {
		printf("Out of room for query strings");
		return;
	}
	query_string_ptr[i++]=query_string;

}

char** parse_bml(char* filename) {
	FILE *file;
	int i=0,j=0,preparse=1;
	static char buf[2048*16];
	static char var[256];
	init_cgi();

	query_string_ptr=malloc(257 * sizeof(char*));


	if (!filename) {
		printf("Error: parse_bml passed a null filename\n");
		return 0;
	}

	file=open_file(filename);

	if (!file)  {
		printf("Error: could not open %s\n",filename);
		return 0;
	}

	while ( (buf[i++]=fgetc(file)) != EOF) {
		if ( (i > 8) && 
			(buf[i - 9] == '<') &&
			(buf[i - 8] == '!') &&
			(buf[i - 7] == '-') &&
			(buf[i - 6] == '-') &&
			(buf[i - 5] == '#') &&
			(buf[i - 4] == 'b') &&
			(buf[i - 3] == 'e') &&
			(buf[i - 2] == 'a') &&
			(buf[i - 1] == 'r') ) {

				buf[i-9]=0; i=0;
				add_query_string(strcpy(malloc(strlen(buf)+1), buf));

				while ( (buf[i++]=fgetc(file)) != EOF ) {
					if ( ( i > 2 ) && 
						 (buf[i-3] == '-') &&
						 (buf[i-2] == '-') &&
						 (buf[i-1] == '>') ) {

						buf[i-3]=0; i=0; 
						/*parse(buf); */
						add_query_string(strcpy(malloc(strlen(buf)+1), buf));
						break;
					}

					if (buf[i-1] == '%') {
						j=0;
						buf[i-1]=0;
						while ( (var[j++]=fgetc(file)) != EOF ) 
							if ((var[j-1] == '%') || (j > 200)) break;
						var[j-1]=0;
						if (get_data(var)) {
							strncat(buf, get_data(var), 200);
						}
						i = strlen (buf);

					}
				}

				
					if ( i ) {
						printf("Error: Tag '<!--#bear' not terminated\n");
						return ;
					}
				
		    }	
		
	}


	if ( i ) {
		buf[--i]=0;
		add_query_string(strcpy(malloc(strlen(buf)+1), buf));
	}
	add_query_string(0);
	add_query_string(0);

	/* When constructing bml files, you may find the parser is not doing
	 * what you want... typically this is because of a malformed file.
	 * enable parse_debug top have the program display it's parsing's
	 */

	if (get_data("parse_debug")) {
		i=0;
		while (query_string_ptr[i]) 
			printf("%d (%s) - '%s'\n",i,(i%2)?"html":"bear",
					query_string_ptr[i++]);
	}

	return query_string_ptr;

}

